@@ -29,6 +29,9 @@ enum {
     ANNOTS_Y2_COLUMN,
     ANNOTS_TYPE_COLUMN,
     ANNOTS_COLOR_COLUMN,
+    ANNOTS_FLAG_INVISIBLE_COLUMN,
+    ANNOTS_FLAG_HIDDEN_COLUMN,
+    ANNOTS_FLAG_PRINT_COLUMN,
     ANNOTS_COLUMN,
     N_COLUMNS
 };
@@ -368,6 +371,106 @@ pgd_annot_view_set_annot_free_text (GtkWidget            *table,
     g_free (text);
 }
 
+static void
+pgd_annots_file_attachment_save_dialog_response (GtkFileChooser    *file_chooser,
+						 gint               response,
+						 PopplerAttachment *attachment)
+{
+    gchar  *filename;
+    GError *error = NULL;
+
+    if (response != GTK_RESPONSE_ACCEPT) {
+        g_object_unref (attachment);
+	gtk_widget_destroy (GTK_WIDGET (file_chooser));
+	return;
+    }
+
+    filename = gtk_file_chooser_get_filename (file_chooser);
+    if (!poppler_attachment_save (attachment, filename, &error)) {
+        g_warning ("%s", error->message);
+	g_error_free (error);
+    }
+    g_free (filename);
+    g_object_unref (attachment);
+    gtk_widget_destroy (GTK_WIDGET (file_chooser));
+}
+
+static void
+pgd_annot_save_file_attachment_button_clicked (GtkButton                  *button,
+					       PopplerAnnotFileAttachment *annot)
+{
+    GtkWidget         *file_chooser;
+    PopplerAttachment *attachment;
+
+    attachment = poppler_annot_file_attachment_get_attachment (annot);
+    if (!attachment)
+        return;
+
+    file_chooser = gtk_file_chooser_dialog_new ("Save attachment",
+						GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (button))),
+						GTK_FILE_CHOOSER_ACTION_SAVE,
+						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+						NULL);
+    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser), attachment->name);
+    g_signal_connect (G_OBJECT (file_chooser), "response",
+		      G_CALLBACK (pgd_annots_file_attachment_save_dialog_response),
+		      (gpointer) attachment);
+    gtk_widget_show (file_chooser);
+}
+
+static void
+pgd_annot_view_set_annot_file_attachment (GtkWidget                  *table,
+					  PopplerAnnotFileAttachment *annot,
+					  gint                       *row)
+{
+    GtkWidget *button;
+    gchar *text;
+
+    text = poppler_annot_file_attachment_get_name (annot);
+    pgd_table_add_property (GTK_TABLE (table), "<b>Attachment Name:</b>", text, row);
+    g_free (text);
+
+    button = gtk_button_new_with_label ("Save Attachment");
+    g_signal_connect (G_OBJECT (button), "clicked",
+		      G_CALLBACK (pgd_annot_save_file_attachment_button_clicked),
+		      (gpointer)annot);
+    pgd_table_add_property_with_custom_widget (GTK_TABLE (table), "<b>File Attachment:</b>", button, row);
+    gtk_widget_show (button);
+
+}
+
+static void
+pgd_annot_view_set_annot_movie (GtkWidget         *table,
+				PopplerAnnotMovie *annot,
+				gint              *row)
+{
+    GtkWidget *movie_view;
+    gchar *text;
+
+    text = poppler_annot_movie_get_title (annot);
+    pgd_table_add_property (GTK_TABLE (table), "<b>Movie Title:</b>", text, row);
+    g_free (text);
+
+    movie_view = pgd_movie_view_new ();
+    pgd_movie_view_set_movie (movie_view, poppler_annot_movie_get_movie (annot));
+    pgd_table_add_property_with_custom_widget (GTK_TABLE (table), "<b>Movie:</b>", movie_view, row);
+    gtk_widget_show (movie_view);
+}
+
+static void
+pgd_annot_view_set_annot_screen (GtkWidget          *table,
+				 PopplerAnnotScreen *annot,
+				 gint               *row)
+{
+    GtkWidget *action_view;
+
+    action_view = pgd_action_view_new (NULL);
+    pgd_action_view_set_action (action_view, poppler_annot_screen_get_action (annot));
+    pgd_table_add_property_with_custom_widget (GTK_TABLE (table), "<b>Action:</b>", action_view, row);
+    gtk_widget_show (action_view);
+}
+
 static void
 pgd_annot_view_set_annot (GtkWidget    *annot_view,
                           PopplerAnnot *annot)
@@ -416,6 +519,10 @@ pgd_annot_view_set_annot (GtkWidget    *annot_view,
     pgd_table_add_property (GTK_TABLE (table), "<b>Flags:</b>", text, &row);
     g_free (text);
 
+    text = g_strdup_printf ("%d", poppler_annot_get_page_index (annot));
+    pgd_table_add_property (GTK_TABLE (table), "<b>Page:</b>", text, &row);
+    g_free (text);
+
     if (POPPLER_IS_ANNOT_MARKUP (annot))
         pgd_annot_view_set_annot_markup (table, POPPLER_ANNOT_MARKUP (annot), &row);
 
@@ -427,6 +534,15 @@ pgd_annot_view_set_annot (GtkWidget    *annot_view,
         case POPPLER_ANNOT_FREE_TEXT:
           pgd_annot_view_set_annot_free_text (table, POPPLER_ANNOT_FREE_TEXT (annot), &row);
           break;
+        case POPPLER_ANNOT_FILE_ATTACHMENT:
+	  pgd_annot_view_set_annot_file_attachment (table, POPPLER_ANNOT_FILE_ATTACHMENT (annot), &row);
+	  break;
+        case POPPLER_ANNOT_MOVIE:
+	  pgd_annot_view_set_annot_movie (table, POPPLER_ANNOT_MOVIE (annot), &row);
+	  break;
+        case POPPLER_ANNOT_SCREEN:
+	  pgd_annot_view_set_annot_screen (table, POPPLER_ANNOT_SCREEN (annot), &row);
+	  break;
         default:
           break;
     }
@@ -478,6 +594,7 @@ pgd_annots_get_annots (GtkWidget     *button,
         GtkTreeIter          iter;
         gchar               *x1, *y1, *x2, *y2;
         GdkPixbuf           *pixbuf;
+	PopplerAnnotFlag     flags;
 
         amapping = (PopplerAnnotMapping *) l->data;
 
@@ -487,15 +604,19 @@ pgd_annots_get_annots (GtkWidget     *button,
         y2 = g_strdup_printf ("%.2f", amapping->area.y2);
 
         pixbuf = get_annot_color (amapping->annot);
+	flags = poppler_annot_get_flags (amapping->annot);
 
         gtk_list_store_append (demo->model, &iter);
         gtk_list_store_set (demo->model, &iter,
-                            ANNOTS_X1_COLUMN, x1, 
+                            ANNOTS_X1_COLUMN, x1,
                             ANNOTS_Y1_COLUMN, y1,
                             ANNOTS_X2_COLUMN, x2,
                             ANNOTS_Y2_COLUMN, y2,
                             ANNOTS_TYPE_COLUMN, get_annot_type (amapping->annot),
                             ANNOTS_COLOR_COLUMN, pixbuf,
+			    ANNOTS_FLAG_INVISIBLE_COLUMN, (flags & POPPLER_ANNOT_FLAG_INVISIBLE),
+			    ANNOTS_FLAG_HIDDEN_COLUMN, (flags & POPPLER_ANNOT_FLAG_HIDDEN),
+			    ANNOTS_FLAG_PRINT_COLUMN, (flags & POPPLER_ANNOT_FLAG_PRINT),
                             ANNOTS_COLUMN, amapping->annot,
                            -1);
 
@@ -606,7 +727,9 @@ pgd_annots_create_widget (PopplerDocument *document)
     demo->model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
-                                      GDK_TYPE_PIXBUF, G_TYPE_OBJECT);
+                                      GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN,
+				      G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
+				      G_TYPE_OBJECT);
     treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (demo->model));
 
     renderer = gtk_cell_renderer_text_new ();
@@ -648,6 +771,27 @@ pgd_annots_create_widget (PopplerDocument *document)
                                                  "pixbuf", ANNOTS_COLOR_COLUMN,
                                                  NULL);
 
+    renderer = gtk_cell_renderer_toggle_new ();
+    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
+                                                 ANNOTS_FLAG_INVISIBLE_COLUMN, "Invisible",
+                                                 renderer,
+                                                 "active", ANNOTS_FLAG_INVISIBLE_COLUMN,
+                                                 NULL);
+
+    renderer = gtk_cell_renderer_toggle_new ();
+    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
+                                                 ANNOTS_FLAG_HIDDEN_COLUMN, "Hidden",
+                                                 renderer,
+                                                 "active", ANNOTS_FLAG_HIDDEN_COLUMN,
+                                                 NULL);
+
+    renderer = gtk_cell_renderer_toggle_new ();
+    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
+                                                 ANNOTS_FLAG_PRINT_COLUMN, "Print",
+                                                 renderer,
+                                                 "active", ANNOTS_FLAG_PRINT_COLUMN,
+                                                 NULL);
+
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
     g_signal_connect (G_OBJECT (selection), "changed",
                       G_CALLBACK (pgd_annots_selection_changed),
