@@ -17,6 +17,7 @@
  */
 
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <poppler.h>
 #include <string.h>
 
@@ -30,6 +31,11 @@
 #include "transitions.h"
 #include "images.h"
 #include "annots.h"
+#include "attachments.h"
+#include "layers.h"
+#include "text.h"
+#include "find.h"
+#include "print.h"
 
 enum {
 	PGD_TITLE_COLUMN,
@@ -53,7 +59,12 @@ static const PopplerGlibDemo demo_list[] = {
 	{ "Forms",            pgd_forms_create_widget },
 	{ "Page Transitions", pgd_transitions_create_widget },
 	{ "Images",           pgd_images_create_widget },
-	{ "Annots",           pgd_annots_create_widget }
+	{ "Annots",           pgd_annots_create_widget },
+	{ "Attachments",      pgd_attachments_create_widget },
+	{ "Layers",           pgd_layers_create_widget },
+	{ "Text",             pgd_text_create_widget },
+	{ "Find",             pgd_find_create_widget },
+	{ "Print",             pgd_print_create_widget }
 };
 
 static void
@@ -147,20 +158,46 @@ gint main (gint argc, gchar **argv)
 		return 1;
 	}
 
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
 	gtk_init (&argc, &argv);
 
-	if (g_ascii_strncasecmp (argv[1], "file://", strlen ("file://")) == 0) {
-		uri = g_strdup (argv[1]);
-	} else {
+#if GLIB_CHECK_VERSION (2,15,0)
+	GFile *file;
+
+	file = g_file_new_for_commandline_arg (argv[1]);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
+#else
+	if (g_path_is_absolute (argv[1])) {
 		uri = g_filename_to_uri (argv[1], NULL, &error);
-		if (error) {
-			g_print ("Error: %s\n", error->message);
-			g_error_free (error);
-		
-			return 1;
-		}
+	} else if (g_ascii_strncasecmp (argv[1], "file://", strlen ("file://")) == 0) {
+		uri = g_strdup (argv[1]);
+	} else if (!g_strrstr (argv[1], "://")) {
+		gchar *dir;
+		gchar *filename;
+
+		dir = g_get_current_dir ();
+		filename = g_build_filename (dir, argv[1], NULL);
+		g_free (dir);
+
+		uri = g_filename_to_uri (filename, NULL, &error);
+		g_free (filename);
+	} else {
+		g_print ("Error: unsupported uri\n");
+
+		return 1;
 	}
-	
+
+	if (error) {
+		g_print ("Error: %s\n", error->message);
+		g_error_free (error);
+
+		return 1;
+	}
+#endif /* GLIB_CHECK_VERSION */
+
 	timer = g_timer_new ();
 	document = poppler_document_new_from_file (uri, NULL, &error);
 	g_timer_stop (timer);
