@@ -116,17 +116,8 @@ _poppler_document_new_from_pdfdoc (PDFDoc  *newDoc,
 
   document->doc = newDoc;
 
-#if defined (HAVE_CAIRO)
   document->output_dev = new CairoOutputDev ();
   document->output_dev->startDoc(document->doc->getXRef (), document->doc->getCatalog ());
-#elif defined (HAVE_SPLASH)
-  SplashColor white;
-  white[0] = 255;
-  white[1] = 255;
-  white[2] = 255;
-  document->output_dev = new SplashOutputDev(splashModeRGB8, 4, gFalse, white);
-  document->output_dev->startDoc(document->doc->getXRef ());
-#endif
 
   return document;
 }
@@ -488,7 +479,7 @@ poppler_document_get_attachments (PopplerDocument *document)
         delete emb_file;
 	continue;
       }
-      attachment = _poppler_attachment_new (document, emb_file);
+      attachment = _poppler_attachment_new (emb_file);
       delete emb_file;
 
       retval = g_list_prepend (retval, attachment);
@@ -976,7 +967,7 @@ poppler_index_iter_copy (PopplerIndexIter *iter)
 
 	g_return_val_if_fail (iter != NULL, NULL);
 
-	new_iter = g_new0 (PopplerIndexIter, 1);
+	new_iter = g_slice_new (PopplerIndexIter);
 	*new_iter = *iter;
 	new_iter->document = (PopplerDocument *) g_object_ref (new_iter->document);
 
@@ -1037,7 +1028,7 @@ poppler_index_iter_new (PopplerDocument *document)
 	if (items == NULL)
 		return NULL;
 
-	iter = g_new0 (PopplerIndexIter, 1);
+	iter = g_slice_new (PopplerIndexIter);
 	iter->document = (PopplerDocument *) g_object_ref (document);
 	iter->items = items;
 	iter->index = 0;
@@ -1067,7 +1058,7 @@ poppler_index_iter_get_child (PopplerIndexIter *parent)
 	if (! (item->hasKids() && item->getKids()) )
 		return NULL;
 
-	child = g_new0 (PopplerIndexIter, 1);
+	child = g_slice_new0 (PopplerIndexIter);
 	child->document = (PopplerDocument *)g_object_ref (parent->document);
 	child->items = item->getKids ();
 
@@ -1185,8 +1176,7 @@ poppler_index_iter_free (PopplerIndexIter *iter)
 		return;
 
 	g_object_unref (iter->document);
-	g_free (iter);
-	
+	g_slice_free (PopplerIndexIter, iter);
 }
 
 struct _PopplerFontsIter
@@ -1302,7 +1292,7 @@ poppler_fonts_iter_copy (PopplerFontsIter *iter)
 
 	g_return_val_if_fail (iter != NULL, NULL);
 
-	new_iter = g_new0 (PopplerFontsIter, 1);
+	new_iter = g_slice_new (PopplerFontsIter);
 	*new_iter = *iter;
 
 	new_iter->items = new GooList ();
@@ -1322,7 +1312,7 @@ poppler_fonts_iter_free (PopplerFontsIter *iter)
 
 	deleteGooList (iter->items, FontInfo);
 
-	g_free (iter);
+	g_slice_free (PopplerFontsIter, iter);
 }
 
 static PopplerFontsIter *
@@ -1330,7 +1320,7 @@ poppler_fonts_iter_new (GooList *items)
 {
 	PopplerFontsIter *iter;
 
-	iter = g_new0 (PopplerFontsIter, 1);
+	iter = g_slice_new (PopplerFontsIter);
 	iter->items = items;
 	iter->index = 0;
 
@@ -1427,7 +1417,7 @@ layer_new (OptionalContentGroup *oc)
 {
   Layer *layer;
 
-  layer = g_new0 (Layer, 1);
+  layer = g_slice_new0 (Layer);
   layer->oc = oc;
 
   return layer;
@@ -1448,7 +1438,7 @@ layer_free (Layer *layer)
 	  g_free (layer->label);
   }
 
-  g_free (layer);
+  g_slice_free (Layer, layer);
 }
 
 static GList *
@@ -1497,8 +1487,8 @@ get_optional_content_rbgroups (OCGs *ocg)
   return groups;
 }
 
-static GList *
-poppler_document_get_layer_rbgroup (PopplerDocument *document,
+GList *
+_poppler_document_get_layer_rbgroup (PopplerDocument *document,
 				    Layer           *layer)
 {
   GList *l;
@@ -1581,8 +1571,8 @@ get_optional_content_items (OCGs *ocg)
   return items;
 }
 
-static GList *
-poppler_document_get_layers (PopplerDocument *document)
+GList *
+_poppler_document_get_layers (PopplerDocument *document)
 {
   if (!document->layers) {
     Catalog *catalog = document->doc->getCatalog ();
@@ -1641,7 +1631,7 @@ poppler_layers_iter_copy (PopplerLayersIter *iter)
 
   g_return_val_if_fail (iter != NULL, NULL);
   
-  new_iter = g_new0 (PopplerLayersIter, 1);
+  new_iter = g_slice_new (PopplerLayersIter);
   *new_iter = *iter;
   new_iter->document = (PopplerDocument *) g_object_ref (new_iter->document);
   
@@ -1661,7 +1651,7 @@ poppler_layers_iter_free (PopplerLayersIter *iter)
     return;
 
   g_object_unref (iter->document);
-  g_free (iter);
+  g_slice_free (PopplerLayersIter, iter);
 }
 
 /**
@@ -1673,12 +1663,12 @@ poppler_layers_iter_new (PopplerDocument *document)
   PopplerLayersIter *iter;
   GList *items;
 
-  items = poppler_document_get_layers (document);
+  items = _poppler_document_get_layers (document);
 
   if (!items)
     return NULL;
 
-  iter = g_new0 (PopplerLayersIter, 1);
+  iter = g_slice_new0 (PopplerLayersIter);
   iter->document = (PopplerDocument *)g_object_ref (document);
   iter->items = items;
 
@@ -1706,7 +1696,7 @@ poppler_layers_iter_get_child (PopplerLayersIter *parent)
   if (!layer || !layer->kids)
     return NULL;
 
-  child = g_new0 (PopplerLayersIter, 1);
+  child = g_slice_new0 (PopplerLayersIter);
   child->document = (PopplerDocument *)g_object_ref (parent->document);
   child->items = layer->kids;
 
@@ -1758,7 +1748,7 @@ poppler_layers_iter_get_layer (PopplerLayersIter *iter)
   if (layer->oc) {
     GList *rb_group = NULL;
 
-    rb_group = poppler_document_get_layer_rbgroup (iter->document, layer);
+    rb_group = _poppler_document_get_layer_rbgroup (iter->document, layer);
     poppler_layer = _poppler_layer_new (iter->document, layer, rb_group);
   }
   
