@@ -71,15 +71,8 @@ poppler_page_finalize (GObject *object)
 
   if (page->annots != NULL)
     delete page->annots;
-#if defined (HAVE_CAIRO)
   if (page->text != NULL) 
     page->text->decRefCnt();
-#else
-  if (page->gfx != NULL)
-    delete page->gfx;  
-  if (page->text_dev != NULL)
-    delete page->text_dev;
-#endif
   /* page->page is owned by the document */
 }
 
@@ -235,107 +228,36 @@ poppler_page_get_transition (PopplerPage *page)
   return transition;
 }
 
-#if !defined (HAVE_CAIRO)
-static TextOutputDev *
-poppler_page_get_text_output_dev (PopplerPage *page)
-{
-  if (page->text_dev == NULL) {
-    page->text_dev = new TextOutputDev (NULL, gTrue, gFalse, gFalse);
-
-    if (page->gfx)
-      delete page->gfx;
-    page->gfx = page->page->createGfx(page->text_dev,
-				      72.0, 72.0, 0,
-				      gFalse, /* useMediaBox */
-				      gTrue, /* Crop */
-				      -1, -1, -1, -1,
-				      gFalse, /* printing */
-				      page->document->doc->getCatalog (),
-				      NULL, NULL, NULL, NULL);
-
-    page->page->display(page->gfx);
-
-    page->text_dev->endPage();
-  }
-
-  return page->text_dev;
-}
-#endif /* !defined (HAVE_CAIRO) */
-
-#if defined (HAVE_CAIRO)
-
 static TextPage *
 poppler_page_get_text_page (PopplerPage *page)
 {
   if (page->text == NULL) {
-    cairo_t *cr;
-    cairo_surface_t *surface;
+    TextOutputDev *text_dev;
+    Gfx           *gfx;
 
-    surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24, 1, 1);
-    cr = cairo_create (surface);
-    poppler_page_render (page, cr);
-    cairo_destroy (cr);
-    cairo_surface_destroy (surface);
+    text_dev = new TextOutputDev (NULL, gTrue, gFalse, gFalse);
+    gfx = page->page->createGfx(text_dev,
+				72.0, 72.0, 0,
+				gFalse, /* useMediaBox */
+				gTrue, /* Crop */
+				-1, -1, -1, -1,
+				gFalse, /* printing */
+				page->document->doc->getCatalog (),
+				NULL, NULL, NULL, NULL);
+    page->page->display(gfx);
+    text_dev->endPage();
 
+    page->text = text_dev->takeText();
+    delete gfx;
+    delete text_dev;
   }
 
   return page->text;
 }
 
 #ifdef POPPLER_WITH_GDK
-typedef struct {
-  unsigned char *cairo_data;
-  cairo_surface_t *surface;
-  cairo_t *cairo;
-} OutputDevData;
-
-static void
-poppler_page_prepare_output_dev (PopplerPage *page,
-				 double scale,
-				 int rotation,
-				 gboolean transparent,
-				 OutputDevData *output_dev_data)
-{
-  CairoOutputDev *output_dev;
-  cairo_surface_t *surface;
-  double width, height;
-  int cairo_width, cairo_height, cairo_rowstride, rotate;
-  unsigned char *cairo_data;
-
-  rotate = rotation + page->page->getRotate ();
-  if (rotate == 90 || rotate == 270) {
-    height = page->page->getCropWidth ();
-    width = page->page->getCropHeight ();
-  } else {
-    width = page->page->getCropWidth ();
-    height = page->page->getCropHeight ();
-  }
-
-  cairo_width = (int) ceil(width * scale);
-  cairo_height = (int) ceil(height * scale);
-
-  output_dev = page->document->output_dev;
-  cairo_rowstride = cairo_width * 4;
-  cairo_data = (guchar *) gmallocn (cairo_height, cairo_rowstride);
-  if (transparent)
-      memset (cairo_data, 0x00, cairo_height * cairo_rowstride);
-  else
-      memset (cairo_data, 0xff, cairo_height * cairo_rowstride);
-
-  surface = cairo_image_surface_create_for_data(cairo_data,
-						CAIRO_FORMAT_ARGB32,
-	  					cairo_width, cairo_height, 
-						cairo_rowstride);
-
-  output_dev_data->cairo_data = cairo_data;
-  output_dev_data->surface = surface;
-  output_dev_data->cairo = cairo_create (surface);
-  output_dev->setCairo (output_dev_data->cairo);
-}
-
 static void
 copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
-			      unsigned char   *data,
 			      GdkPixbuf       *pixbuf)
 {
   int cairo_width, cairo_height, cairo_rowstride;
@@ -346,8 +268,8 @@ copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
 
   cairo_width = cairo_image_surface_get_width (surface);
   cairo_height = cairo_image_surface_get_height (surface);
-  cairo_rowstride = cairo_width * 4;
-  cairo_data = data;
+  cairo_rowstride = cairo_image_surface_get_stride (surface);
+  cairo_data = cairo_image_surface_get_data (surface);
 
   pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
   pixbuf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
@@ -373,152 +295,8 @@ copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
 	}
     }
 }	
-
-static void
-poppler_page_copy_to_pixbuf (PopplerPage *page,
-			     GdkPixbuf *pixbuf,
-			     OutputDevData *output_dev_data)
-{
-  copy_cairo_surface_to_pixbuf (output_dev_data->surface,
-				output_dev_data->cairo_data,
-				pixbuf);
-  
-  page->document->output_dev->setCairo (NULL);
-  cairo_surface_destroy (output_dev_data->surface);
-  cairo_destroy (output_dev_data->cairo);
-  gfree (output_dev_data->cairo_data);
-}
-
-static void
-poppler_page_set_selection_alpha (PopplerPage           *page,
-				  double                 scale,
-				  GdkPixbuf             *pixbuf,
-				  PopplerSelectionStyle  style,
-				  PopplerRectangle      *selection)
-{
-  /* Cairo doesn't need this, since cairo generates an alpha channel. */ 
-}
 #endif /* POPPLER_WITH_GDK */
 
-#elif defined (HAVE_SPLASH) && defined (POPPLER_WITH_GDK)
- 
-typedef struct {
-} OutputDevData;
-
-static void
-poppler_page_prepare_output_dev (PopplerPage *page,
-				 double scale,
-				 int rotation,
-				 gboolean transparent,
-				 OutputDevData *output_dev_data)
-{
-  /* pft */
-}
-
-static void
-poppler_page_copy_to_pixbuf(PopplerPage *page,
-			    GdkPixbuf *pixbuf,
-			    OutputDevData *data)
-{
-  SplashOutputDev *output_dev;
-  SplashBitmap *bitmap;
-  SplashColorPtr color_ptr;
-  int splash_width, splash_height, splash_rowstride;
-  int pixbuf_rowstride, pixbuf_n_channels;
-  guchar *pixbuf_data, *dst;
-  int x, y;
-
-  output_dev = page->document->output_dev;
-
-  bitmap = output_dev->getBitmap ();
-  color_ptr = bitmap->getDataPtr ();
-
-  splash_width = bitmap->getWidth ();
-  splash_height = bitmap->getHeight ();
-  splash_rowstride = bitmap->getRowSize ();
-
-  pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
-  pixbuf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  pixbuf_n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-
-  if (splash_width > gdk_pixbuf_get_width (pixbuf))
-    splash_width = gdk_pixbuf_get_width (pixbuf);
-  if (splash_height > gdk_pixbuf_get_height (pixbuf))
-    splash_height = gdk_pixbuf_get_height (pixbuf);
-
-  SplashColorPtr pixel = new Guchar[4];
-  for (y = 0; y < splash_height; y++)
-  {
-    dst = pixbuf_data + y * pixbuf_rowstride;
-    for (x = 0; x < splash_width; x++)
-    {
-      output_dev->getBitmap()->getPixel(x, y, pixel);
-      dst[0] = pixel[0];
-      dst[1] = pixel[1];
-      dst[2] = pixel[2];
-      if (pixbuf_n_channels == 4)
-        dst[3] = 0xff;
-      dst += pixbuf_n_channels;
-    }
-  }
-  delete [] pixel;
-}
-
-static void
-poppler_page_set_selection_alpha (PopplerPage           *page,
-				  double                 scale,
-				  GdkPixbuf             *pixbuf,
-				  PopplerSelectionStyle  style,
-				  PopplerRectangle      *selection)
-{
-  GList *region, *l;
-  gint x, y, width, height;
-  int pixbuf_rowstride, pixbuf_n_channels;
-  guchar *pixbuf_data, *dst;
-
-  pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
-  pixbuf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  pixbuf_n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-  width = gdk_pixbuf_get_width (pixbuf);
-  height = gdk_pixbuf_get_height (pixbuf);
-
-  if (pixbuf_n_channels != 4)
-    return;
-
-  for (y = 0; y < height; y++) {
-      dst = pixbuf_data + y * pixbuf_rowstride;
-      for (x = 0; x < width; x++) {
-	  dst[3] = 0x00;
-	  dst += pixbuf_n_channels;
-      }
-  }
-
-  region = poppler_page_get_selection_region (page, scale, style, selection);
-
-  for (l = region; l; l = g_list_next (l)) {
-    PopplerRectangle *rectangle = (PopplerRectangle *)l->data;
-    GdkRectangle rect;
-    
-    rect.x = (gint)rectangle->x1;
-    rect.y = (gint)rectangle->y1;
-    rect.width  = (gint) (rectangle->x2 - rectangle->x1);
-    rect.height = (gint) (rectangle->y2 - rectangle->y1);
-    
-    for (y = 0; y < rect.height; y++) {
-      dst = pixbuf_data + (rect.y + y) * pixbuf_rowstride +
-	rect.x * pixbuf_n_channels;
-      for (x = 0; x < rect.width; x++) {
-	  dst[3] = 0xff;
-	  dst += pixbuf_n_channels;
-      }
-    }
-  }
-
-  poppler_page_selection_region_free (region);
-}
-
-#endif /* HAVE_SPLASH */
-
 static GBool
 poppler_print_annot_cb (Annot *annot, void *user_data)
 {
@@ -527,8 +305,6 @@ poppler_print_annot_cb (Annot *annot, void *user_data)
   return (annot->getType() == Annot::typeWidget);
 }
 
-#if defined (HAVE_CAIRO)
-
 static void
 _poppler_page_render (PopplerPage *page,
 		      cairo_t *cairo,
@@ -609,28 +385,28 @@ create_surface_from_thumbnail_data (guchar *data,
 				    gint    rowstride)
 {
   guchar *cairo_pixels;
+  gint cairo_stride;
   cairo_surface_t *surface;
-  static cairo_user_data_key_t key;
   int j;
 
-  cairo_pixels = (guchar *)g_malloc (4 * width * height);
-  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
-						 CAIRO_FORMAT_RGB24,
-						 width, height, 4 * width);
-  cairo_surface_set_user_data (surface, &key,
-			       cairo_pixels, (cairo_destroy_func_t)g_free);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24, width, height);
+  if (cairo_surface_status (surface))
+    return NULL;
+
+  cairo_pixels = cairo_image_surface_get_data (surface);
+  cairo_stride = cairo_image_surface_get_stride (surface);
 
   for (j = height; j; j--) {
     guchar *p = data;
     guchar *q = cairo_pixels;
     guchar *end = p + 3 * width;
-	  
+
     while (p < end) {
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
       q[0] = p[2];
       q[1] = p[1];
       q[2] = p[0];
-#else	  
+#else
       q[1] = p[0];
       q[2] = p[1];
       q[3] = p[2];
@@ -640,7 +416,7 @@ create_surface_from_thumbnail_data (guchar *data,
     }
 
     data += rowstride;
-    cairo_pixels += 4 * width;
+    cairo_pixels += cairo_stride;
   }
 
   return surface;
@@ -749,8 +525,6 @@ poppler_page_render_selection (PopplerPage           *page,
   output_dev->setCairo (NULL);
 }
 
-#endif /* HAVE_CAIRO */
-
 #ifdef POPPLER_WITH_GDK
 static void
 _poppler_page_render_to_pixbuf (PopplerPage *page,
@@ -761,23 +535,47 @@ _poppler_page_render_to_pixbuf (PopplerPage *page,
 				GBool printing,
 				GdkPixbuf *pixbuf)
 {
-  OutputDevData data;
-  
-  poppler_page_prepare_output_dev (page, scale, rotation, FALSE, &data);
+  cairo_t *cr;
+  cairo_surface_t *surface;
 
-  page->page->displaySlice(page->document->output_dev,
-			   72.0 * scale, 72.0 * scale,
-			   rotation,
-			   gFalse, /* useMediaBox */
-			   gTrue, /* Crop */
-			   src_x, src_y,
-			   src_width, src_height,
-			   printing,
-			   page->document->doc->getCatalog (),
-			   NULL, NULL,
-			   printing ? poppler_print_annot_cb : NULL, NULL);
-  
-  poppler_page_copy_to_pixbuf (page, pixbuf, &data);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					src_width, src_height);
+  cr = cairo_create (surface);
+  cairo_save (cr);
+  switch (rotation) {
+  case 90:
+	  cairo_translate (cr, src_x + src_width, -src_y);
+	  break;
+  case 180:
+	  cairo_translate (cr, src_x + src_width, src_y + src_height);
+	  break;
+  case 270:
+	  cairo_translate (cr, -src_x, src_y + src_height);
+	  break;
+  default:
+	  cairo_translate (cr, -src_x, -src_y);
+  }
+
+  if (scale != 1.0)
+	  cairo_scale (cr, scale, scale);
+
+  if (rotation != 0)
+	  cairo_rotate (cr, rotation * G_PI / 180.0);
+
+  if (printing)
+	  poppler_page_render_for_printing (page, cr);
+  else
+	  poppler_page_render (page, cr);
+  cairo_restore (cr);
+
+  cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OVER);
+  cairo_set_source_rgb (cr, 1., 1., 1.);
+  cairo_paint (cr);
+
+  cairo_destroy (cr);
+
+  copy_cairo_surface_to_pixbuf (surface, pixbuf);
+  cairo_surface_destroy (surface);
 }
 
 /**
@@ -911,71 +709,64 @@ poppler_page_render_selection_to_pixbuf (PopplerPage           *page,
                                          GdkColor              *glyph_color,
                                          GdkColor              *background_color)
 {
-  OutputDev *output_dev;
-  OutputDevData data;
-  SelectionStyle selection_style = selectionStyleGlyph;
-  PDFRectangle pdf_selection(selection->x1, selection->y1,
-			     selection->x2, selection->y2);
+  cairo_t *cr;
+  cairo_surface_t *surface;
+  double width, height;
+  int cairo_width, cairo_height, rotate;
+  PopplerColor poppler_background_color;
+  PopplerColor poppler_glyph_color;
 
-  GfxColor gfx_background_color = { 
-      {
-	  background_color->red,
-	  background_color->green,
-	  background_color->blue
-      }
-  };
-  GfxColor gfx_glyph_color = {
-      {
-	  glyph_color->red,
-	  glyph_color->green,
-	  glyph_color->blue
-      }
-  };
+  poppler_background_color.red = background_color->red;
+  poppler_background_color.green = background_color->green;
+  poppler_background_color.blue = background_color->blue;
+  poppler_glyph_color.red = glyph_color->red;
+  poppler_glyph_color.green = glyph_color->green;
+  poppler_glyph_color.blue = glyph_color->blue;
 
-  switch (style)
-    {
-      case POPPLER_SELECTION_GLYPH:
-        selection_style = selectionStyleGlyph;
-	break;
-      case POPPLER_SELECTION_WORD:
-        selection_style = selectionStyleWord;
-	break;
-      case POPPLER_SELECTION_LINE:
-        selection_style = selectionStyleLine;
-	break;
-    }
+  rotate = rotation + page->page->getRotate ();
+  if (rotate == 90 || rotate == 270) {
+    height = page->page->getCropWidth ();
+    width = page->page->getCropHeight ();
+  } else {
+    width = page->page->getCropWidth ();
+    height = page->page->getCropHeight ();
+  }
 
-  output_dev = page->document->output_dev;
+  cairo_width = (int) ceil(width * scale);
+  cairo_height = (int) ceil(height * scale);
 
-  poppler_page_prepare_output_dev (page, scale, rotation, TRUE, &data);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					cairo_width, cairo_height);
+  cr = cairo_create (surface);
+  cairo_set_source_rgba (cr, 0, 0, 0, 0);
+  cairo_paint (cr);
 
-#if defined (HAVE_CAIRO)
-  TextPage *text;
+  switch (rotate) {
+  case 90:
+	  cairo_translate (cr, cairo_width, 0);
+	  break;
+  case 180:
+	  cairo_translate (cr, cairo_width, cairo_height);
+	  break;
+  case 270:
+	  cairo_translate (cr, 0, cairo_height);
+	  break;
+  default:
+	  cairo_translate (cr, 0, 0);
+  }
+  if (scale != 1.0)
+	  cairo_scale (cr, scale, scale);
 
-  text = poppler_page_get_text_page (page);
-  text->drawSelection (output_dev, scale, rotation,
-			   &pdf_selection, selection_style,
-			   &gfx_glyph_color, &gfx_background_color);
-#else
-  TextOutputDev *text_dev;
+  if (rotate != 0)
+	  cairo_rotate (cr, rotation * G_PI / 180.0);
 
-  text_dev = poppler_page_get_text_output_dev (page);
-  text_dev->drawSelection (output_dev, scale, rotation,
-			   &pdf_selection, selection_style,
-			   &gfx_glyph_color, &gfx_background_color);
-  /* We'll need a function to destroy page->text_dev and page->gfx
-   * when the application wants to get rid of them.
-   *
-   * Two improvements: 1) make GfxFont refcounted and let TextPage and
-   * friends hold a reference to the GfxFonts they need so we can free
-   * up Gfx early.  2) use a TextPage directly when rendering the page
-   * so we don't have to use TextOutputDev and render a second
-   * time. */
-#endif
-  
-  poppler_page_copy_to_pixbuf (page, pixbuf, &data);
+  poppler_page_render_selection (page, cr, selection, old_selection, style,
+				 &poppler_glyph_color, &poppler_background_color);
 
-  poppler_page_set_selection_alpha (page, scale, pixbuf, style, selection);
+  cairo_destroy (cr);
+
+  copy_cairo_surface_to_pixbuf (surface, pixbuf);
+  cairo_surface_destroy (surface);
 }
 
 #endif /* POPPLER_WITH_GDK */
@@ -1048,6 +839,7 @@ poppler_page_get_selection_region (PopplerPage           *page,
 				   PopplerRectangle      *selection)
 {
   PDFRectangle poppler_selection;
+  TextPage *text;
   SelectionStyle selection_style = selectionStyleGlyph;
   GooList *list;
   GList *region = NULL;
@@ -1071,20 +863,10 @@ poppler_page_get_selection_region (PopplerPage           *page,
 	break;
     }
 
-#if defined (HAVE_CAIRO)
-  TextPage *text;
-
   text = poppler_page_get_text_page (page);
   list = text->getSelectionRegion(&poppler_selection,
 				  selection_style, scale);
-#else
-  TextOutputDev *text_dev;
-  
-  text_dev = poppler_page_get_text_output_dev (page);
-  list = text_dev->getSelectionRegion(&poppler_selection,
-				      selection_style, scale);
-#endif
-  
+
   for (i = 0; i < list->getLength(); i++) {
     PDFRectangle *selection_rect = (PDFRectangle *) list->get(i);
     PopplerRectangle *rect;
@@ -1135,6 +917,7 @@ poppler_page_get_text (PopplerPage          *page,
   GooString *sel_text;
   double height;
   char *result;
+  TextPage *text;
   SelectionStyle selection_style = selectionStyleGlyph;
   PDFRectangle pdf_selection;
 
@@ -1160,18 +943,8 @@ poppler_page_get_text (PopplerPage          *page,
 	break;
     }
 
-#if defined (HAVE_CAIRO)
-  TextPage *text;
-
   text = poppler_page_get_text_page (page);
   sel_text = text->getSelectionText (&pdf_selection, selection_style);
-#else
-  TextOutputDev *text_dev;
-
-  text_dev = poppler_page_get_text_output_dev (page);
-  sel_text = text_dev->getSelectionText (&pdf_selection, selection_style);
-#endif
-	  
   result = g_strdup (sel_text->getCString ());
   delete sel_text;
 
@@ -1198,23 +971,12 @@ poppler_page_find_text (PopplerPage *page,
   gunichar *ucs4;
   glong ucs4_len;
   double height;
-#if defined (HAVE_CAIRO)
   TextPage *text_dev;
-#else
-  TextOutputDev *text_dev;
-#endif
-  
+
   g_return_val_if_fail (POPPLER_IS_PAGE (page), FALSE);
   g_return_val_if_fail (text != NULL, FALSE);
 
-#if defined (HAVE_CAIRO)
   text_dev = poppler_page_get_text_page (page);
-#else
-  text_dev = new TextOutputDev (NULL, gTrue, gFalse, gFalse);
-  page->page->display (text_dev, 72, 72, 0,
-		       gFalse, gTrue, gFalse,
-		       page->document->doc->getCatalog());
-#endif
 
   ucs4 = g_utf8_to_ucs4_fast (text, -1, &ucs4_len);
   poppler_page_get_size (page, NULL, &height);
@@ -1237,17 +999,11 @@ poppler_page_find_text (PopplerPage *page,
       matches = g_list_prepend (matches, match);
     }
 
-#if !defined (HAVE_CAIRO)
-  delete text_dev;
-#endif
-  
   g_free (ucs4);
 
   return g_list_reverse (matches);
 }
 
-#if defined (HAVE_CAIRO)
-
 static CairoImageOutputDev *
 poppler_page_get_image_output_dev (PopplerPage *page,
 				   GBool (*imgDrawDeviceCbk)(int img_id, void *data),
@@ -1388,21 +1144,6 @@ poppler_page_free_image_mapping (GList *list)
   g_list_free (list);
 }
 
-#else /* HAVE_CAIRO */
-
-GList *
-poppler_page_get_image_mapping (PopplerPage *page)
-{
-  return NULL;
-}
-
-void
-poppler_page_free_image_mapping (GList *list)
-{
-}
-
-#endif /* HAVE_CAIRO */
-
 /**
  * poppler_page_render_to_ps:
  * @page: a #PopplerPage
@@ -1518,7 +1259,7 @@ poppler_page_get_link_mapping (PopplerPage *page)
       /* Create the mapping */
       mapping = g_new (PopplerLinkMapping, 1);
       mapping->action = _poppler_action_new (page->document, link_action, NULL);
-      
+
       link->getRect (&rect.x1, &rect.y1, &rect.x2, &rect.y2);
       
       switch (page->page->getRotate ())
@@ -1700,6 +1441,15 @@ poppler_page_get_annot_mapping (PopplerPage *page)
       case Annot::typeFreeText:
         mapping->annot = _poppler_annot_free_text_new (annot);
 	break;
+      case Annot::typeFileAttachment:
+        mapping->annot = _poppler_annot_file_attachment_new (annot);
+	break;
+      case Annot::typeMovie:
+        mapping->annot = _poppler_annot_movie_new (annot);
+	break;
+      case Annot::typeScreen:
+        mapping->annot = _poppler_annot_screen_new (annot);
+	break;
       default:
         mapping->annot = _poppler_annot_new (annot);
 	break;
