@@ -5,13 +5,15 @@
 // This file is licensed under the GPLv2 or later
 //
 // Copyright 2006-2008 Julien Rebetez <julienr@svn.gnome.org>
-// Copyright 2007-2009 Albert Astals Cid <aacid@kde.org>
+// Copyright 2007-2010 Albert Astals Cid <aacid@kde.org>
 // Copyright 2007-2008 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright 2007 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2007 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright 2008 Pino Toscano <pino@kde.org>
 // Copyright 2008 Michael Vrable <mvrable@cs.ucsd.edu>
-//
+// Copyright 2009 Matthias Drochner <M.Drochner@fz-juelich.de>
+// Copyright 2009 KDAB via Guillermo Amaral <gamaral@amaral.com.mx>
+// 
 //========================================================================
 
 #include <config.h>
@@ -118,17 +120,6 @@ FormWidget::FormWidget(XRef *xrefA, Object *aobj, unsigned num, Ref aref, FormFi
     obj1.free();
 }
 
-FormWidget::FormWidget(FormWidget *dest)
-{
-  x1 = dest->x1;
-  y1 = dest->y1;
-  x2 = dest->x2;
-  y2 = dest->x2;
-
-  type = dest->type;
-  field = dest->field;
-}
-
 FormWidget::~FormWidget()
 {
   obj.free ();
@@ -276,6 +267,7 @@ void FormWidgetButton::loadDefaults ()
     //We didn't found the "on" state for the button
     if (!onStr) {
       error(-1, "FormWidgetButton:: unable to find the on state for the button\n");
+      onStr = new GooString(""); // TODO is this the best solution?
     }
   }
 
@@ -460,7 +452,9 @@ void FormWidgetChoice::loadDefaults ()
         obj3.free();
         obj4.free();
       } else {
-        error(-1, "FormWidgetChoice:: invalid Opt entry\n");
+        error(-1, "FormWidgetChoice:: invalid %d Opt entry\n", i);
+        parent->_setChoiceExportVal(i, new GooString(""));
+        parent->_setChoiceOptionName(i, new GooString(""));
       }
       obj2.free();
     }
@@ -544,7 +538,7 @@ void FormWidgetChoice::_updateV ()
     } else if (numSelected == 1) {
       for(int i=0; i<parent->getNumChoices(); i++) {
         if (parent->isSelected(i)) {
-          obj1.initString(new GooString(parent->getExportVal(i)));
+          obj1.initString(new GooString(parent->getChoice(i)));
           break;
         }
       }
@@ -553,7 +547,7 @@ void FormWidgetChoice::_updateV ()
       for(int i=0; i<parent->getNumChoices(); i++) {
         if (parent->isSelected(i)) {
           Object obj2;
-          obj2.initString(new GooString(parent->getExportVal(i)));
+          obj2.initString(new GooString(parent->getChoice(i)));
           obj1.arrayAdd(&obj2);
         }
       }
@@ -813,7 +807,7 @@ void FormField::fillChildrenSiblingsID()
 {
   if(terminal) return;
   for (int i=0; i<numChildren; i++) {
-    children[i]->loadChildrenDefaults();
+    children[i]->fillChildrenSiblingsID();
   }
 }
 
@@ -996,7 +990,7 @@ FormFieldText::FormFieldText(XRef *xrefA, Object *aobj, const Ref& ref)
 GooString* FormFieldText::getContentCopy ()
 {
   if (!content) return NULL;
-  return new GooString(*content);
+  return new GooString(content);
 }
 
 void FormFieldText::setContentCopy (GooString* new_content)
