@@ -1,9 +1,9 @@
 /* poppler-page.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2009, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2010, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2005, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
- * Copyright (C) 2006-2009, Pino Toscano <pino@kde.org>
+ * Copyright (C) 2006-2010, Pino Toscano <pino@kde.org>
  * Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2009 Shawn Rutledge <shawn.t.rutledge@gmail.com>
  *
@@ -38,10 +38,10 @@
 #include <TextOutputDev.h>
 #include <Annot.h>
 #include <Link.h>
+#include <ArthurOutputDev.h>
 #if defined(HAVE_SPLASH)
 #include <SplashOutputDev.h>
 #include <splash/SplashBitmap.h>
-#include <ArthurOutputDev.h>
 #endif
 
 #include "poppler-private.h"
@@ -242,7 +242,6 @@ QImage Page::renderToImage(double xres, double yres, int x, int y, int w, int h,
     }
     case Poppler::Document::ArthurBackend:
     {
-#if defined(HAVE_SPLASH)
       QSize size = pageSize();
       QImage tmpimg(w == -1 ? qRound( size.width() * xres / 72.0 ) : w, h == -1 ? qRound( size.height() * yres / 72.0 ) : h, QImage::Format_ARGB32);
 
@@ -251,7 +250,6 @@ QImage Page::renderToImage(double xres, double yres, int x, int y, int w, int h,
           painter.setRenderHint(QPainter::Antialiasing);
       if (m_page->parentDoc->m_hints & Document::TextAntialiasing)
           painter.setRenderHint(QPainter::TextAntialiasing);
-      painter.save();
       painter.translate(x == -1 ? 0 : -x, y == -1 ? 0 : -y);
       ArthurOutputDev arthur_output(&painter);
       arthur_output.startDoc(m_page->parentDoc->doc->getXRef());
@@ -267,10 +265,8 @@ QImage Page::renderToImage(double xres, double yres, int x, int y, int w, int h,
 						 y,
 						 w,
 						 h);
-      painter.restore();
       painter.end();
       img = tmpimg;
-#endif
       break;
     }
   }
@@ -323,7 +319,7 @@ QString Page::text(const QRectF &r) const
   return result;
 }
 
-bool Page::search(const QString &text, QRectF &rect, SearchDirection direction, SearchMode caseSensitive, Rotation rotate) const
+bool Page::search(const QString &text, double &sLeft, double &sTop, double &sRight, double &sBottom, SearchDirection direction, SearchMode caseSensitive, Rotation rotate) const
 {
   const QChar * str = text.unicode();
   int len = text.length();
@@ -335,11 +331,6 @@ bool Page::search(const QString &text, QRectF &rect, SearchDirection direction,
   else sCase = gFalse;
 
   bool found = false;
-  double sLeft, sTop, sRight, sBottom;
-  sLeft = rect.left();
-  sTop = rect.top();
-  sRight = rect.right();
-  sBottom = rect.bottom();
 
   int rotation = (int)rotate * 90;
 
@@ -360,6 +351,19 @@ bool Page::search(const QString &text, QRectF &rect, SearchDirection direction,
 
   textPage->decRefCnt();
 
+  return found;
+}
+
+bool Page::search(const QString &text, QRectF &rect, SearchDirection direction, SearchMode caseSensitive, Rotation rotate) const
+{
+  double sLeft, sTop, sRight, sBottom;
+  sLeft = rect.left();
+  sTop = rect.top();
+  sRight = rect.right();
+  sBottom = rect.bottom();
+
+  bool found = search(text, sLeft, sTop, sRight, sBottom, direction, caseSensitive, rotate);
+
   rect.setLeft( sLeft );
   rect.setTop( sTop );
   rect.setRight( sRight );
