@@ -11,7 +11,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005-2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2010 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Marco Pesenti Gritti <mpg@redhat.com>
 //
 // To see a description of the changes please see the Changelog file that
@@ -27,6 +27,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include "goo/gmem.h"
 #include "SplashErrorCodes.h"
 #include "SplashMath.h"
@@ -2001,6 +2002,9 @@ SplashError Splash::fillImageMask(SplashImageMaskSource src, void *srcData,
   xq = w % scaledWidth;
 
   // allocate pixel buffer
+  if (yp < 0 || yp > INT_MAX - 1) {
+    return splashErrBadArg;
+  }
   pixBuf = (SplashColorPtr)gmallocn((yp + 1), w);
 
   // initialize the pixel pipe
@@ -2301,6 +2305,9 @@ SplashError Splash::drawImage(SplashImageSource src, void *srcData,
   xq = w % scaledWidth;
 
   // allocate pixel buffers
+  if (yp < 0 || yp > INT_MAX - 1) {
+    return splashErrBadArg;
+  }
   colorBuf = (SplashColorPtr)gmallocn3((yp + 1), w, nComps);
   if (srcAlpha) {
     alphaBuf = (Guchar *)gmallocn((yp + 1), w);
@@ -3011,11 +3018,11 @@ SplashError Splash::composite(SplashBitmap *src, int xSrc, int ySrc,
       pipeSetXY(&pipe, xDest, yDest + y);
       ap = src->getAlphaPtr() + (ySrc + y) * src->getWidth() + xSrc;
       for (x = 0; x < w; ++x) {
-	src->getPixel(xSrc + x, ySrc + y, pixel);
 	alpha = *ap++;
 	if (noClip || state->clip->test(xDest + x, yDest + y)) {
 	  // this uses shape instead of alpha, which isn't technically
 	  // correct, but works out the same
+	  src->getPixel(xSrc + x, ySrc + y, pixel);
 	  pipe.shape = (SplashCoord)(alpha / 255.0);
 	  pipeRun(&pipe);
 	  updateModX(xDest + x);
@@ -3031,8 +3038,8 @@ SplashError Splash::composite(SplashBitmap *src, int xSrc, int ySrc,
     for (y = 0; y < h; ++y) {
       pipeSetXY(&pipe, xDest, yDest + y);
       for (x = 0; x < w; ++x) {
-	src->getPixel(xSrc + x, ySrc + y, pixel);
 	if (noClip || state->clip->test(xDest + x, yDest + y)) {
+	  src->getPixel(xSrc + x, ySrc + y, pixel);
 	  pipeRun(&pipe);
 	  updateModX(xDest + x);
 	  updateModY(yDest + y);
@@ -3156,7 +3163,7 @@ void Splash::compositeBackground(SplashColorPtr color) {
 SplashError Splash::blitTransparent(SplashBitmap *src, int xSrc, int ySrc,
 				    int xDest, int yDest, int w, int h) {
   SplashColor pixel;
-  SplashColorPtr p;
+  SplashColorPtr p, sp;
   Guchar *q;
   int x, y, mask;
 
@@ -3196,23 +3203,24 @@ SplashError Splash::blitTransparent(SplashBitmap *src, int xSrc, int ySrc,
   case splashModeBGR8:
     for (y = 0; y < h; ++y) {
       p = &bitmap->data[(yDest + y) * bitmap->rowSize + 3 * xDest];
+      sp = &src->data[(ySrc + y) * src->rowSize + 3 * xSrc];
       for (x = 0; x < w; ++x) {
-	src->getPixel(xSrc + x, ySrc + y, pixel);
-	*p++ = pixel[0];
-	*p++ = pixel[1];
-	*p++ = pixel[2];
+	*p++ = *sp++;
+	*p++ = *sp++;
+	*p++ = *sp++;
       }
     }
     break;
   case splashModeXBGR8:
     for (y = 0; y < h; ++y) {
       p = &bitmap->data[(yDest + y) * bitmap->rowSize + 4 * xDest];
+      sp = &src->data[(ySrc + y) * src->rowSize + 4 * xSrc];
       for (x = 0; x < w; ++x) {
-	src->getPixel(xSrc + x, ySrc + y, pixel);
-	*p++ = pixel[0];
-	*p++ = pixel[1];
-	*p++ = pixel[2];
+	*p++ = *sp++;
+	*p++ = *sp++;
+	*p++ = *sp++;
 	*p++ = 255;
+	*sp++;
       }
     }
     break;
