@@ -14,7 +14,8 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dom Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -42,6 +43,7 @@
 #include "Catalog.h"
 #include "Page.h"
 #include "PDFDoc.h"
+#include "PDFDocFactory.h"
 #include "CharTypes.h"
 #include "UnicodeMap.h"
 #include "PDFDocEncoding.h"
@@ -158,15 +160,13 @@ int main(int argc, char *argv[]) {
     userPW = NULL;
   }
 
-  if(fileName->cmp("-") != 0) {
-      doc = new PDFDoc(fileName, ownerPW, userPW);
-  } else {
-      Object obj;
-
-      obj.initNull();
-      doc = new PDFDoc(new FileStream(stdin, 0, gFalse, 0, &obj), ownerPW, userPW);
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
   }
 
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
   if (userPW) {
     delete userPW;
   }
@@ -303,7 +303,7 @@ int main(int argc, char *argv[]) {
   printf("Optimized:      %s\n", doc->isLinearized() ? "yes" : "no");
 
   // print PDF version
-  printf("PDF version:    %.1f\n", doc->getPDFVersion());
+  printf("PDF version:    %d.%d\n", doc->getPDFMajorVersion(), doc->getPDFMinorVersion());
 
   // print the metadata
   if (printMetadata && (metadata = doc->readMetadata())) {
@@ -319,6 +319,7 @@ int main(int argc, char *argv[]) {
  err2:
   uMap->decRefCnt();
   delete doc;
+  delete fileName;
  err1:
   delete globalParams;
  err0:
