@@ -8,6 +8,23 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006 Dominic Lachowicz <cinamod@hotmail.com>
+// Copyright (C) 2007-2008, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009 Jan Jockusch <jan@jockusch.de>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include "config.h"
 #include <poppler-config.h>
 #include <stdio.h>
@@ -15,6 +32,7 @@
 #include <stddef.h>
 #include <string.h>
 #include "parseargs.h"
+#include "printencodings.h"
 #include "goo/GooString.h"
 #include "goo/gmem.h"
 #include "GlobalParams.h"
@@ -26,11 +44,11 @@
 #include "Catalog.h"
 #include "Page.h"
 #include "PDFDoc.h"
+#include "PDFDocFactory.h"
 #include "TextOutputDev.h"
 #include "CharTypes.h"
 #include "UnicodeMap.h"
 #include "Error.h"
-#include "UGooString.h"
 
 static void printInfoString(FILE *f, Dict *infoDict, char *key,
 			    char *text1, char *text2, UnicodeMap *uMap);
@@ -38,6 +56,11 @@ static void printInfoDate(FILE *f, Dict *infoDict, char *key, char *fmt);
 
 static int firstPage = 1;
 static int lastPage = 0;
+static double resolution = 72.0;
+static int x = 0;
+static int y = 0;
+static int w = 0;
+static int h = 0;
 static GBool physLayout = gFalse;
 static GBool rawOrder = gFalse;
 static GBool htmlMeta = gFalse;
@@ -47,15 +70,25 @@ static GBool noPageBreaks = gFalse;
 static char ownerPassword[33] = "\001";
 static char userPassword[33] = "\001";
 static GBool quiet = gFalse;
-static char cfgFileName[256] = "";
 static GBool printVersion = gFalse;
 static GBool printHelp = gFalse;
+static GBool printEnc = gFalse;
 
-static ArgDesc argDesc[] = {
+static const ArgDesc argDesc[] = {
   {"-f",       argInt,      &firstPage,     0,
    "first page to convert"},
   {"-l",       argInt,      &lastPage,      0,
    "last page to convert"},
+  {"-r",      argFP,       &resolution,    0,
+   "resolution, in DPI (default is 72)"},
+  {"-x",      argInt,      &x,             0,
+   "x-coordinate of the crop area top left corner"},
+  {"-y",      argInt,      &y,             0,
+   "y-coordinate of the crop area top left corner"},
+  {"-W",      argInt,      &w,             0,
+   "width of crop area in pixels (default is 0)"},
+  {"-H",      argInt,      &h,             0,
+   "height of crop area in pixels (default is 0)"},
   {"-layout",  argFlag,     &physLayout,    0,
    "maintain original physical layout"},
   {"-raw",     argFlag,     &rawOrder,      0,
@@ -64,6 +97,8 @@ static ArgDesc argDesc[] = {
    "generate a simple HTML file, including the meta information"},
   {"-enc",     argString,   textEncName,    sizeof(textEncName),
    "output text encoding name"},
+  {"-listenc",argFlag,     &printEnc,      0,
+   "list available encodings"},
   {"-eol",     argString,   textEOL,        sizeof(textEOL),
    "output end-of-line convention (unix, dos, or mac)"},
   {"-nopgbrk", argFlag,     &noPageBreaks,  0,
@@ -74,8 +109,6 @@ static ArgDesc argDesc[] = {
    "user password (for encrypted files)"},
   {"-q",       argFlag,     &quiet,         0,
    "don't print any messages or errors"},
-  {"-cfg",     argString,   cfgFileName,    sizeof(cfgFileName),
-   "configuration file to use in place of .xpdfrc"},
   {"-v",       argFlag,     &printVersion,  0,
    "print copyright and version info"},
   {"-h",       argFlag,     &printHelp,     0,
@@ -106,18 +139,27 @@ int main(int argc, char *argv[]) {
 
   // parse args
   ok = parseArgs(argDesc, &argc, argv);
-  if (!ok || argc < 2 || argc > 3 || printVersion || printHelp) {
-    fprintf(stderr, "pdftotext version %s\n", xpdfVersion);
+  if (!ok || (argc < 2 && !printEnc) || argc > 3 || printVersion || printHelp) {
+    fprintf(stderr, "pdftotext version %s\n", PACKAGE_VERSION);
+    fprintf(stderr, "%s\n", popplerCopyright);
     fprintf(stderr, "%s\n", xpdfCopyright);
     if (!printVersion) {
       printUsage("pdftotext", "<PDF-file> [<text-file>]", argDesc);
     }
     goto err0;
   }
-  fileName = new GooString(argv[1]);
 
   // read config file
-  globalParams = new GlobalParams(cfgFileName);
+  globalParams = new GlobalParams();
+
+  if (printEnc) {
+    printEncodings();
+    delete globalParams;
+    goto err0;
+  }
+
+  fileName = new GooString(argv[1]);
+
   if (textEncName[0]) {
     globalParams->setTextEncoding(textEncName);
   }
@@ -151,7 +193,14 @@ int main(int argc, char *argv[]) {
   } else {
     userPW = NULL;
   }
-  doc = new PDFDoc(fileName, ownerPW, userPW);
+
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
+  }
+
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
   if (userPW) {
     delete userPW;
   }
@@ -175,6 +224,9 @@ int main(int argc, char *argv[]) {
   // construct text file name
   if (argc == 3) {
     textFileName = new GooString(argv[2]);
+  } else if (fileName->cmp("fd://0") == 0) {
+     error(-1, "You have to provide an output filename when reading form stdin.");
+     goto err2;
   } else {
     p = fileName->getCString() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
@@ -239,8 +291,19 @@ int main(int argc, char *argv[]) {
   textOut = new TextOutputDev(textFileName->getCString(),
 			      physLayout, rawOrder, htmlMeta);
   if (textOut->isOk()) {
-      doc->displayPages(textOut, firstPage, lastPage, 72, 72, 0,
+    if ((w==0) && (h==0) && (x==0) && (y==0)) {
+      doc->displayPages(textOut, firstPage, lastPage, resolution, resolution, 0,
 			gTrue, gFalse, gFalse);
+    } else {
+      int page;
+      
+      for (page = firstPage; page <= lastPage; ++page) {
+	doc->displayPageSlice(textOut, page, resolution, resolution, 0,
+			      gTrue, gFalse, gFalse, 
+			      x, y, w, h);
+      }	
+    }	
+
   } else {
     delete textOut;
     exitCode = 2;
@@ -274,6 +337,7 @@ int main(int argc, char *argv[]) {
   delete textFileName;
  err2:
   delete doc;
+  delete fileName;
   uMap->decRefCnt();
  err1:
   delete globalParams;
